# -*- coding: utf-8 -*-


def has_period(self):
    """Indicates if an axis has symmetries.
    Parameters
    ----------
    self: DataPattern
        a DataPatternobject
    Returns
    -------
    Boolean
    """

    return False
